/*-
 * Copyright (c) 2015 Taylor R. Campbell
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <errno.h>
#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "attribute.h"
#include "progname.h"

#include "err.h"

void attr_noreturn
err(int status, const char *fmt, ...)
{
	int error = errno;
	va_list va;

	(void)fprintf(stderr, "%s: ", getprogname());

	va_start(va, fmt);
	(void)vfprintf(stderr, fmt, va);
	va_end(va);

	(void)fprintf(stderr, ": %s\n", strerror(error));

	exit(status);
}

void attr_noreturn
errx(int status, const char *fmt, ...)
{
	va_list va;

	(void)fprintf(stderr, "%s: ", getprogname());

	va_start(va, fmt);
	(void)vfprintf(stderr, fmt, va);
	va_end(va);

	(void)fprintf(stderr, "\n");

	exit(status);
}

void
warnx(const char *fmt, ...)
{
	va_list va;

	(void)fprintf(stderr, "%s: ", getprogname());

	va_start(va, fmt);
	(void)vfprintf(stderr, fmt, va);
	va_end(va);

	(void)fprintf(stderr, "\n");
}
